@@ -1,10 +1,3 @@
-{-# OPTIONS -w #-}
--- The above warning supression flag is a temporary kludge.
--- While working on this module you are encouraged to remove it and fix
--- any warnings in the module. See
---     http://hackage.haskell.org/trac/ghc/wiki/Commentary/CodingStyle#Warnings
--- for details
-
 -----------------------------------------------------------------------------
 --
 -- Cmm optimisation
@@ -24,25 +17,23 @@ module CmmOpt (
 #include "HsVersions.h"
 
 import OldCmm
+import OldPprCmm
 import CmmNode (wrapRecExp)
 import CmmUtils
-import CLabel
 import StaticFlags
 
 import UniqFM
 import Unique
+import Util
 import FastTypes
 import Outputable
+import Platform
 import BlockId
 
 import Data.Bits
-import Data.Word
-import Data.Int
 import Data.Maybe
 import Data.List
 
-import Compiler.Hoopl hiding (Unique)
-
 -- -----------------------------------------------------------------------------
 -- Eliminates dead blocks
 
@@ -68,7 +59,7 @@ cmmEliminateDeadBlocks blocks@(BasicBlock base_id _:_) =
                 stmt m (CmmComment _) = m
                 stmt m (CmmAssign _ e) = expr m e
                 stmt m (CmmStore e1 e2) = expr (expr m e1) e2
-                stmt m (CmmCall c _ as _ _) = f (actuals m as) c
+                stmt m (CmmCall c _ as _) = f (actuals m as) c
                     where f m (CmmCallee e _) = expr m e
                           f m (CmmPrim _) = m
                 stmt m (CmmBranch b) = b:m
@@ -108,11 +99,11 @@ works as follows:
 
   - count uses of each temporary
   - for each temporary:
-	- attempt to push it forward to the statement that uses it
+        - attempt to push it forward to the statement that uses it
         - only push forward past assignments to other temporaries
-	  (assumes that temporaries are single-assignment)
-	- if we reach the statement that uses it, inline the rhs
-	  and delete the original assignment.
+          (assumes that temporaries are single-assignment)
+        - if we reach the statement that uses it, inline the rhs
+          and delete the original assignment.
 
 [N.B. In the Quick C-- compiler, this optimization is achieved by a
  combination of two dataflow passes: forward substitution (peephole
@@ -155,57 +146,53 @@ countUses :: UserOfLocalRegs a => a -> UniqFM Int
 countUses a = foldRegsUsed (\m r -> addToUFM m r (count m r + 1)) emptyUFM a
   where count m r = lookupWithDefaultUFM m (0::Int) r
 
-cmmMiniInline :: [CmmBasicBlock] -> [CmmBasicBlock]
-cmmMiniInline blocks = map do_inline blocks 
+cmmMiniInline :: Platform -> [CmmBasicBlock] -> [CmmBasicBlock]
+cmmMiniInline platform blocks = map do_inline blocks
   where do_inline (BasicBlock id stmts)
-          = BasicBlock id (cmmMiniInlineStmts (countUses blocks) stmts)
+          = BasicBlock id (cmmMiniInlineStmts platform (countUses blocks) stmts)
 
-cmmMiniInlineStmts :: UniqFM Int -> [CmmStmt] -> [CmmStmt]
-cmmMiniInlineStmts uses [] = []
-cmmMiniInlineStmts uses (stmt@(CmmAssign (CmmLocal (LocalReg u _)) expr) : stmts)
+cmmMiniInlineStmts :: Platform -> UniqFM Int -> [CmmStmt] -> [CmmStmt]
+cmmMiniInlineStmts _        _    [] = []
+cmmMiniInlineStmts platform uses (stmt@(CmmAssign (CmmLocal (LocalReg u _)) expr) : stmts)
         -- not used: just discard this assignment
   | Nothing <- lookupUFM uses u
-  = cmmMiniInlineStmts uses stmts
+  = cmmMiniInlineStmts platform uses stmts
 
         -- used (literal): try to inline at all the use sites
   | Just n <- lookupUFM uses u, isLit expr
   =
-#ifdef NCG_DEBUG
-     trace ("nativeGen: inlining " ++ showSDoc (pprStmt stmt)) $
-#endif
+     ncgDebugTrace ("nativeGen: inlining " ++ showSDoc (pprStmt platform stmt)) $
      case lookForInlineLit u expr stmts of
          (m, stmts')
-             | n == m -> cmmMiniInlineStmts (delFromUFM uses u) stmts'
+             | n == m -> cmmMiniInlineStmts platform (delFromUFM uses u) stmts'
              | otherwise ->
-                 stmt : cmmMiniInlineStmts (adjustUFM (\x -> x - m) uses u) stmts'
+                 stmt : cmmMiniInlineStmts platform (adjustUFM (\x -> x - m) uses u) stmts'
 
         -- used (foldable to literal): try to inline at all the use sites
   | Just n <- lookupUFM uses u,
     e@(CmmLit _) <- wrapRecExp foldExp expr
   =
-#ifdef NCG_DEBUG
-     trace ("nativeGen: inlining " ++ showSDoc (pprStmt stmt)) $
-#endif
+     ncgDebugTrace ("nativeGen: inlining " ++ showSDoc (pprStmt platform stmt)) $
      case lookForInlineLit u e stmts of
          (m, stmts')
-             | n == m -> cmmMiniInlineStmts (delFromUFM uses u) stmts'
+             | n == m -> cmmMiniInlineStmts platform (delFromUFM uses u) stmts'
              | otherwise ->
-                 stmt : cmmMiniInlineStmts (adjustUFM (\x -> x - m) uses u) stmts'
+                 stmt : cmmMiniInlineStmts platform (adjustUFM (\x -> x - m) uses u) stmts'
 
         -- used once (non-literal): try to inline at the use site
   | Just 1 <- lookupUFM uses u,
     Just stmts' <- lookForInline u expr stmts
   = 
-#ifdef NCG_DEBUG
-     trace ("nativeGen: inlining " ++ showSDoc (pprStmt stmt)) $
-#endif
-     cmmMiniInlineStmts uses stmts'
+     ncgDebugTrace ("nativeGen: inlining " ++ showSDoc (pprStmt platform stmt)) $
+     cmmMiniInlineStmts platform uses stmts'
  where
-  foldExp (CmmMachOp op args) = cmmMachOpFold op args
+  foldExp (CmmMachOp op args) = cmmMachOpFold platform op args
   foldExp e = e
 
-cmmMiniInlineStmts uses (stmt:stmts)
-  = stmt : cmmMiniInlineStmts uses stmts
+  ncgDebugTrace str x = if ncgDebugIsOn then trace str x else x
+
+cmmMiniInlineStmts platform uses (stmt:stmts)
+  = stmt : cmmMiniInlineStmts platform uses stmts
 
 -- | Takes a register, a 'CmmLit' expression assigned to that
 -- register, and a list of statements.  Inlines the expression at all
@@ -229,12 +216,15 @@ lookForInlineLit u expr stmts@(stmt : rest)
     -- We skip over assignments to registers, unless the register
     -- being assigned to is the one we're inlining.
     ok_to_skip = case stmt of
-        CmmAssign (CmmLocal r@(LocalReg u' _)) _ | u' == u -> False
+        CmmAssign (CmmLocal (LocalReg u' _)) _ | u' == u -> False
         _other -> True
 
+lookForInline :: Unique -> CmmExpr -> [CmmStmt] -> Maybe [CmmStmt]
 lookForInline u expr stmts = lookForInline' u expr regset stmts
     where regset = foldRegsUsed extendRegSet emptyRegSet expr
 
+lookForInline' :: Unique -> CmmExpr -> RegSet -> [CmmStmt] -> Maybe [CmmStmt]
+lookForInline' _ _    _      [] = panic "lookForInline' []"
 lookForInline' u expr regset (stmt : rest)
   | Just 1 <- lookupUFM (countUses stmt) u, ok_to_inline
   = Just (inlineStmt u expr stmt : rest)
@@ -248,14 +238,14 @@ lookForInline' u expr regset (stmt : rest)
   = Nothing
 
   where
-	-- we don't inline into CmmCall if the expression refers to global
-	-- registers.  This is a HACK to avoid global registers clashing with
-	-- C argument-passing registers, really the back-end ought to be able
-	-- to handle it properly, but currently neither PprC nor the NCG can
-	-- do it.  See also CgForeignCall:load_args_into_temps.
+        -- we don't inline into CmmCall if the expression refers to global
+        -- registers.  This is a HACK to avoid global registers clashing with
+        -- C argument-passing registers, really the back-end ought to be able
+        -- to handle it properly, but currently neither PprC nor the NCG can
+        -- do it.  See also CgForeignCall:load_args_into_temps.
     ok_to_inline = case stmt of
-		     CmmCall{} -> hasNoGlobalRegs expr
-		     _ -> True
+                     CmmCall{} -> hasNoGlobalRegs expr
+                     _ -> True
 
    -- Expressions aren't side-effecting.  Temporaries may or may not
    -- be single-assignment depending on the source (the old code
@@ -268,23 +258,23 @@ lookForInline' u expr regset (stmt : rest)
     ok_to_skip = case stmt of
                  CmmNop -> True
                  CmmComment{} -> True
-                 CmmAssign (CmmLocal r@(LocalReg u' _)) rhs | u' /= u && not (r `elemRegSet` regset) -> True
-                 CmmAssign g@(CmmGlobal _) rhs -> not (g `regUsedIn` expr)
+                 CmmAssign (CmmLocal r@(LocalReg u' _)) _rhs | u' /= u && not (r `elemRegSet` regset) -> True
+                 CmmAssign g@(CmmGlobal _) _rhs -> not (g `regUsedIn` expr)
                  _other -> False
 
 
 inlineStmt :: Unique -> CmmExpr -> CmmStmt -> CmmStmt
 inlineStmt u a (CmmAssign r e) = CmmAssign r (inlineExpr u a e)
 inlineStmt u a (CmmStore e1 e2) = CmmStore (inlineExpr u a e1) (inlineExpr u a e2)
-inlineStmt u a (CmmCall target regs es srt ret)
-   = CmmCall (infn target) regs es' srt ret
+inlineStmt u a (CmmCall target regs es ret)
+   = CmmCall (infn target) regs es' ret
    where infn (CmmCallee fn cconv) = CmmCallee (inlineExpr u a fn) cconv
-	 infn (CmmPrim p) = CmmPrim p
-	 es' = [ (CmmHinted (inlineExpr u a e) hint) | (CmmHinted e hint) <- es ]
+         infn (CmmPrim p) = CmmPrim p
+         es' = [ (CmmHinted (inlineExpr u a e) hint) | (CmmHinted e hint) <- es ]
 inlineStmt u a (CmmCondBranch e d) = CmmCondBranch (inlineExpr u a e) d
 inlineStmt u a (CmmSwitch e d) = CmmSwitch (inlineExpr u a e) d
 inlineStmt u a (CmmJump e d) = CmmJump (inlineExpr u a e) d
-inlineStmt u a other_stmt = other_stmt
+inlineStmt _ _ other_stmt = other_stmt
 
 inlineExpr :: Unique -> CmmExpr -> CmmExpr -> CmmExpr
 inlineExpr u a e@(CmmReg (CmmLocal (LocalReg u' _)))
@@ -297,7 +287,7 @@ inlineExpr u a e@(CmmRegOff (CmmLocal (LocalReg u' rep)) off)
     width = typeWidth rep
 inlineExpr u a (CmmLoad e rep) = CmmLoad (inlineExpr u a e) rep
 inlineExpr u a (CmmMachOp op es) = CmmMachOp op (map (inlineExpr u a) es)
-inlineExpr u a other_expr = other_expr
+inlineExpr _ _ other_expr = other_expr
 
 -- -----------------------------------------------------------------------------
 -- MachOp constant folder
@@ -306,41 +296,43 @@ inlineExpr u a other_expr = other_expr
 -- been optimized and folded.
 
 cmmMachOpFold
-    :: MachOp       -- The operation from an CmmMachOp
+    :: Platform
+    -> MachOp       -- The operation from an CmmMachOp
     -> [CmmExpr]    -- The optimized arguments
     -> CmmExpr
 
-cmmMachOpFold op args = fromMaybe (CmmMachOp op args) (cmmMachOpFoldM op args)
+cmmMachOpFold platform op args = fromMaybe (CmmMachOp op args) (cmmMachOpFoldM platform op args)
 
 -- Returns Nothing if no changes, useful for Hoopl, also reduces
 -- allocation!
 cmmMachOpFoldM
-    :: MachOp
+    :: Platform
+    -> MachOp
     -> [CmmExpr]
     -> Maybe CmmExpr
 
-cmmMachOpFoldM op arg@[CmmLit (CmmInt x rep)]
+cmmMachOpFoldM _ op [CmmLit (CmmInt x rep)]
   = Just $ case op of
-      MO_S_Neg r -> CmmLit (CmmInt (-x) rep)
-      MO_Not r   -> CmmLit (CmmInt (complement x) rep)
+      MO_S_Neg _ -> CmmLit (CmmInt (-x) rep)
+      MO_Not _   -> CmmLit (CmmInt (complement x) rep)
 
         -- these are interesting: we must first narrow to the 
         -- "from" type, in order to truncate to the correct size.
         -- The final narrow/widen to the destination type
         -- is implicit in the CmmLit.
-      MO_SF_Conv from to -> CmmLit (CmmFloat (fromInteger x) to)
-      MO_SS_Conv from to -> CmmLit (CmmInt (narrowS from x) to)
-      MO_UU_Conv from to -> CmmLit (CmmInt (narrowU from x) to)
+      MO_SF_Conv _from to -> CmmLit (CmmFloat (fromInteger x) to)
+      MO_SS_Conv  from to -> CmmLit (CmmInt (narrowS from x) to)
+      MO_UU_Conv  from to -> CmmLit (CmmInt (narrowU from x) to)
 
       _ -> panic "cmmMachOpFoldM: unknown unary op"
 
 
 -- Eliminate conversion NOPs
-cmmMachOpFoldM (MO_SS_Conv rep1 rep2) [x] | rep1 == rep2 = Just x
-cmmMachOpFoldM (MO_UU_Conv rep1 rep2) [x] | rep1 == rep2 = Just x
+cmmMachOpFoldM _ (MO_SS_Conv rep1 rep2) [x] | rep1 == rep2 = Just x
+cmmMachOpFoldM _ (MO_UU_Conv rep1 rep2) [x] | rep1 == rep2 = Just x
 
 -- Eliminate nested conversions where possible
-cmmMachOpFoldM conv_outer args@[CmmMachOp conv_inner [x]]
+cmmMachOpFoldM platform conv_outer [CmmMachOp conv_inner [x]]
   | Just (rep1,rep2,signed1) <- isIntConversion conv_inner,
     Just (_,   rep3,signed2) <- isIntConversion conv_outer
   = case () of
@@ -350,13 +342,13 @@ cmmMachOpFoldM conv_outer args@[CmmMachOp conv_inner [x]]
         -- but remember to use the signedness from the widening, just in case
         -- the final conversion is a widen.
         | rep1 < rep2 && rep2 > rep3 ->
-            Just $ cmmMachOpFold (intconv signed1 rep1 rep3) [x]
+            Just $ cmmMachOpFold platform (intconv signed1 rep1 rep3) [x]
         -- Nested widenings: collapse if the signedness is the same
         | rep1 < rep2 && rep2 < rep3 && signed1 == signed2 ->
-            Just $ cmmMachOpFold (intconv signed1 rep1 rep3) [x]
+            Just $ cmmMachOpFold platform (intconv signed1 rep1 rep3) [x]
         -- Nested narrowings: collapse
         | rep1 > rep2 && rep2 > rep3 ->
-            Just $ cmmMachOpFold (MO_UU_Conv rep1 rep3) [x]
+            Just $ cmmMachOpFold platform (MO_UU_Conv rep1 rep3) [x]
         | otherwise ->
             Nothing
   where
@@ -373,22 +365,22 @@ cmmMachOpFoldM conv_outer args@[CmmMachOp conv_inner [x]]
 -- but what if the architecture only supports word-sized loads, should
 -- we do the transformation anyway?
 
-cmmMachOpFoldM mop args@[CmmLit (CmmInt x xrep), CmmLit (CmmInt y _)]
+cmmMachOpFoldM _ mop [CmmLit (CmmInt x xrep), CmmLit (CmmInt y _)]
   = case mop of
         -- for comparisons: don't forget to narrow the arguments before
         -- comparing, since they might be out of range.
-        MO_Eq r   -> Just $ CmmLit (CmmInt (if x_u == y_u then 1 else 0) wordWidth)
-        MO_Ne r   -> Just $ CmmLit (CmmInt (if x_u /= y_u then 1 else 0) wordWidth)
+        MO_Eq _   -> Just $ CmmLit (CmmInt (if x_u == y_u then 1 else 0) wordWidth)
+        MO_Ne _   -> Just $ CmmLit (CmmInt (if x_u /= y_u then 1 else 0) wordWidth)
 
-        MO_U_Gt r -> Just $ CmmLit (CmmInt (if x_u >  y_u then 1 else 0) wordWidth)
-        MO_U_Ge r -> Just $ CmmLit (CmmInt (if x_u >= y_u then 1 else 0) wordWidth)
-        MO_U_Lt r -> Just $ CmmLit (CmmInt (if x_u <  y_u then 1 else 0) wordWidth)
-        MO_U_Le r -> Just $ CmmLit (CmmInt (if x_u <= y_u then 1 else 0) wordWidth)
+        MO_U_Gt _ -> Just $ CmmLit (CmmInt (if x_u >  y_u then 1 else 0) wordWidth)
+        MO_U_Ge _ -> Just $ CmmLit (CmmInt (if x_u >= y_u then 1 else 0) wordWidth)
+        MO_U_Lt _ -> Just $ CmmLit (CmmInt (if x_u <  y_u then 1 else 0) wordWidth)
+        MO_U_Le _ -> Just $ CmmLit (CmmInt (if x_u <= y_u then 1 else 0) wordWidth)
 
-        MO_S_Gt r -> Just $ CmmLit (CmmInt (if x_s >  y_s then 1 else 0) wordWidth)
-        MO_S_Ge r -> Just $ CmmLit (CmmInt (if x_s >= y_s then 1 else 0) wordWidth)
-        MO_S_Lt r -> Just $ CmmLit (CmmInt (if x_s <  y_s then 1 else 0) wordWidth)
-        MO_S_Le r -> Just $ CmmLit (CmmInt (if x_s <= y_s then 1 else 0) wordWidth)
+        MO_S_Gt _ -> Just $ CmmLit (CmmInt (if x_s >  y_s then 1 else 0) wordWidth)
+        MO_S_Ge _ -> Just $ CmmLit (CmmInt (if x_s >= y_s then 1 else 0) wordWidth)
+        MO_S_Lt _ -> Just $ CmmLit (CmmInt (if x_s <  y_s then 1 else 0) wordWidth)
+        MO_S_Le _ -> Just $ CmmLit (CmmInt (if x_s <= y_s then 1 else 0) wordWidth)
 
         MO_Add r -> Just $ CmmLit (CmmInt (x + y) r)
         MO_Sub r -> Just $ CmmLit (CmmInt (x - y) r)
@@ -406,7 +398,7 @@ cmmMachOpFoldM mop args@[CmmLit (CmmInt x xrep), CmmLit (CmmInt y _)]
         MO_U_Shr r -> Just $ CmmLit (CmmInt (x_u `shiftR` fromIntegral y) r)
         MO_S_Shr r -> Just $ CmmLit (CmmInt (x `shiftR` fromIntegral y) r)
 
-        other      -> Nothing
+        _          -> Nothing
 
    where
         x_u = narrowU xrep x
@@ -420,9 +412,9 @@ cmmMachOpFoldM mop args@[CmmLit (CmmInt x xrep), CmmLit (CmmInt y _)]
 -- also assume that constants have been shifted to the right when
 -- possible.
 
-cmmMachOpFoldM op [x@(CmmLit _), y]
+cmmMachOpFoldM platform op [x@(CmmLit _), y]
    | not (isLit y) && isCommutableMachOp op
-   = Just (cmmMachOpFold op [y, x])
+   = Just (cmmMachOpFold platform op [y, x])
 
 -- Turn (a+b)+c into a+(b+c) where possible.  Because literals are
 -- moved to the right, it is more likely that we will find
@@ -440,37 +432,37 @@ cmmMachOpFoldM op [x@(CmmLit _), y]
 -- Also don't do it if arg1 is PicBaseReg, so that we don't separate the
 -- PicBaseReg from the corresponding label (or label difference).
 --
-cmmMachOpFoldM mop1 [CmmMachOp mop2 [arg1,arg2], arg3]
+cmmMachOpFoldM platform mop1 [CmmMachOp mop2 [arg1,arg2], arg3]
    | mop2 `associates_with` mop1
      && not (isLit arg1) && not (isPicReg arg1)
-   = Just (cmmMachOpFold mop2 [arg1, cmmMachOpFold mop1 [arg2,arg3]])
+   = Just (cmmMachOpFold platform mop2 [arg1, cmmMachOpFold platform mop1 [arg2,arg3]])
    where
      MO_Add{} `associates_with` MO_Sub{} = True
      mop1 `associates_with` mop2 =
         mop1 == mop2 && isAssociativeMachOp mop1
 
 -- special case: (a - b) + c  ==>  a + (c - b)
-cmmMachOpFoldM mop1@(MO_Add{}) [CmmMachOp mop2@(MO_Sub{}) [arg1,arg2], arg3]
+cmmMachOpFoldM platform mop1@(MO_Add{}) [CmmMachOp mop2@(MO_Sub{}) [arg1,arg2], arg3]
    | not (isLit arg1) && not (isPicReg arg1)
-   = Just (cmmMachOpFold mop1 [arg1, cmmMachOpFold mop2 [arg3,arg2]])
+   = Just (cmmMachOpFold platform mop1 [arg1, cmmMachOpFold platform mop2 [arg3,arg2]])
 
 -- Make a RegOff if we can
-cmmMachOpFoldM (MO_Add _) [CmmReg reg, CmmLit (CmmInt n rep)]
+cmmMachOpFoldM _ (MO_Add _) [CmmReg reg, CmmLit (CmmInt n rep)]
   = Just $ CmmRegOff reg (fromIntegral (narrowS rep n))
-cmmMachOpFoldM (MO_Add _) [CmmRegOff reg off, CmmLit (CmmInt n rep)]
+cmmMachOpFoldM _ (MO_Add _) [CmmRegOff reg off, CmmLit (CmmInt n rep)]
   = Just $ CmmRegOff reg (off + fromIntegral (narrowS rep n))
-cmmMachOpFoldM (MO_Sub _) [CmmReg reg, CmmLit (CmmInt n rep)]
+cmmMachOpFoldM _ (MO_Sub _) [CmmReg reg, CmmLit (CmmInt n rep)]
   = Just $ CmmRegOff reg (- fromIntegral (narrowS rep n))
-cmmMachOpFoldM (MO_Sub _) [CmmRegOff reg off, CmmLit (CmmInt n rep)]
+cmmMachOpFoldM _ (MO_Sub _) [CmmRegOff reg off, CmmLit (CmmInt n rep)]
   = Just $ CmmRegOff reg (off - fromIntegral (narrowS rep n))
 
 -- Fold label(+/-)offset into a CmmLit where possible
 
-cmmMachOpFoldM (MO_Add _) [CmmLit (CmmLabel lbl), CmmLit (CmmInt i rep)]
+cmmMachOpFoldM _ (MO_Add _) [CmmLit (CmmLabel lbl), CmmLit (CmmInt i rep)]
   = Just $ CmmLit (CmmLabelOff lbl (fromIntegral (narrowU rep i)))
-cmmMachOpFoldM (MO_Add _) [CmmLit (CmmInt i rep), CmmLit (CmmLabel lbl)]
+cmmMachOpFoldM _ (MO_Add _) [CmmLit (CmmInt i rep), CmmLit (CmmLabel lbl)]
   = Just $ CmmLit (CmmLabelOff lbl (fromIntegral (narrowU rep i)))
-cmmMachOpFoldM (MO_Sub _) [CmmLit (CmmLabel lbl), CmmLit (CmmInt i rep)]
+cmmMachOpFoldM _ (MO_Sub _) [CmmLit (CmmLabel lbl), CmmLit (CmmInt i rep)]
   = Just $ CmmLit (CmmLabelOff lbl (fromIntegral (negate (narrowU rep i))))
 
 
@@ -481,18 +473,17 @@ cmmMachOpFoldM (MO_Sub _) [CmmLit (CmmLabel lbl), CmmLit (CmmInt i rep)]
 -- narrowing throws away bits from the operand, there's no way to do
 -- the same comparison at the larger size.
 
-#if i386_TARGET_ARCH || x86_64_TARGET_ARCH
--- powerPC NCG has a TODO for I8/I16 comparisons, so don't try
-
-cmmMachOpFoldM cmp [CmmMachOp conv [x], CmmLit (CmmInt i _)]
-  |     -- if the operand is widened:
+cmmMachOpFoldM platform cmp [CmmMachOp conv [x], CmmLit (CmmInt i _)]
+  |     -- powerPC NCG has a TODO for I8/I16 comparisons, so don't try
+    platformArch platform `elem` [ArchX86, ArchX86_64],
+        -- if the operand is widened:
     Just (rep, signed, narrow_fn) <- maybe_conversion conv,
         -- and this is a comparison operation:
     Just narrow_cmp <- maybe_comparison cmp rep signed,
         -- and the literal fits in the smaller size:
     i == narrow_fn rep i
         -- then we can do the comparison at the smaller size
-  = Just (cmmMachOpFold narrow_cmp [x, CmmLit (CmmInt i rep)])
+  = Just (cmmMachOpFold platform narrow_cmp [x, CmmLit (CmmInt i rep)])
  where
     maybe_conversion (MO_UU_Conv from to)
         | to > from
@@ -523,62 +514,60 @@ cmmMachOpFoldM cmp [CmmMachOp conv [x], CmmLit (CmmInt i _)]
     maybe_comparison (MO_S_Le _) rep False = Just (MO_U_Le rep)
     maybe_comparison _ _ _ = Nothing
 
-#endif
-
 -- We can often do something with constants of 0 and 1 ...
 
-cmmMachOpFoldM mop args@[x, y@(CmmLit (CmmInt 0 _))]
+cmmMachOpFoldM _ mop [x, y@(CmmLit (CmmInt 0 _))]
   = case mop of
-        MO_Add   r -> Just x
-        MO_Sub   r -> Just x
-        MO_Mul   r -> Just y
-        MO_And   r -> Just y
-        MO_Or    r -> Just x
-        MO_Xor   r -> Just x
-        MO_Shl   r -> Just x
-        MO_S_Shr r -> Just x
-        MO_U_Shr r -> Just x
-        MO_Ne    r | isComparisonExpr x -> Just x
-        MO_Eq    r | Just x' <- maybeInvertCmmExpr x -> Just x'
-        MO_U_Gt  r | isComparisonExpr x -> Just x
-        MO_S_Gt  r | isComparisonExpr x -> Just x
-        MO_U_Lt  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
-        MO_S_Lt  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
-        MO_U_Ge  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
-        MO_S_Ge  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
-        MO_U_Le  r | Just x' <- maybeInvertCmmExpr x -> Just x'
-        MO_S_Le  r | Just x' <- maybeInvertCmmExpr x -> Just x'
-        other    -> Nothing
+        MO_Add   _ -> Just x
+        MO_Sub   _ -> Just x
+        MO_Mul   _ -> Just y
+        MO_And   _ -> Just y
+        MO_Or    _ -> Just x
+        MO_Xor   _ -> Just x
+        MO_Shl   _ -> Just x
+        MO_S_Shr _ -> Just x
+        MO_U_Shr _ -> Just x
+        MO_Ne    _ | isComparisonExpr x -> Just x
+        MO_Eq    _ | Just x' <- maybeInvertCmmExpr x -> Just x'
+        MO_U_Gt  _ | isComparisonExpr x -> Just x
+        MO_S_Gt  _ | isComparisonExpr x -> Just x
+        MO_U_Lt  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
+        MO_S_Lt  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
+        MO_U_Ge  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
+        MO_S_Ge  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
+        MO_U_Le  _ | Just x' <- maybeInvertCmmExpr x -> Just x'
+        MO_S_Le  _ | Just x' <- maybeInvertCmmExpr x -> Just x'
+        _ -> Nothing
 
-cmmMachOpFoldM mop args@[x, y@(CmmLit (CmmInt 1 rep))]
+cmmMachOpFoldM _ mop [x, (CmmLit (CmmInt 1 rep))]
   = case mop of
-        MO_Mul    r -> Just x
-        MO_S_Quot r -> Just x
-        MO_U_Quot r -> Just x
-        MO_S_Rem  r -> Just $ CmmLit (CmmInt 0 rep)
-        MO_U_Rem  r -> Just $ CmmLit (CmmInt 0 rep)
-        MO_Ne    r | Just x' <- maybeInvertCmmExpr x -> Just x'
-        MO_Eq    r | isComparisonExpr x -> Just x
-        MO_U_Lt  r | Just x' <- maybeInvertCmmExpr x -> Just x'
-        MO_S_Lt  r | Just x' <- maybeInvertCmmExpr x -> Just x'
-        MO_U_Gt  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
-        MO_S_Gt  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
-        MO_U_Le  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
-        MO_S_Le  r | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
-        MO_U_Ge  r | isComparisonExpr x -> Just x
-        MO_S_Ge  r | isComparisonExpr x -> Just x
-        other       -> Nothing
+        MO_Mul    _ -> Just x
+        MO_S_Quot _ -> Just x
+        MO_U_Quot _ -> Just x
+        MO_S_Rem  _ -> Just $ CmmLit (CmmInt 0 rep)
+        MO_U_Rem  _ -> Just $ CmmLit (CmmInt 0 rep)
+        MO_Ne    _ | Just x' <- maybeInvertCmmExpr x -> Just x'
+        MO_Eq    _ | isComparisonExpr x -> Just x
+        MO_U_Lt  _ | Just x' <- maybeInvertCmmExpr x -> Just x'
+        MO_S_Lt  _ | Just x' <- maybeInvertCmmExpr x -> Just x'
+        MO_U_Gt  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
+        MO_S_Gt  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 0 wordWidth)
+        MO_U_Le  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
+        MO_S_Le  _ | isComparisonExpr x -> Just $ CmmLit (CmmInt 1 wordWidth)
+        MO_U_Ge  _ | isComparisonExpr x -> Just x
+        MO_S_Ge  _ | isComparisonExpr x -> Just x
+        _ -> Nothing
 
 -- Now look for multiplication/division by powers of 2 (integers).
 
-cmmMachOpFoldM mop args@[x, y@(CmmLit (CmmInt n _))]
+cmmMachOpFoldM platform mop [x, (CmmLit (CmmInt n _))]
   = case mop of
         MO_Mul rep
            | Just p <- exactLog2 n ->
-                 Just (cmmMachOpFold (MO_Shl rep) [x, CmmLit (CmmInt p rep)])
+                 Just (cmmMachOpFold platform (MO_Shl rep) [x, CmmLit (CmmInt p rep)])
         MO_U_Quot rep
            | Just p <- exactLog2 n ->
-                 Just (cmmMachOpFold (MO_U_Shr rep) [x, CmmLit (CmmInt p rep)])
+                 Just (cmmMachOpFold platform (MO_U_Shr rep) [x, CmmLit (CmmInt p rep)])
         MO_S_Quot rep
            | Just p <- exactLog2 n, 
              CmmReg _ <- x ->   -- We duplicate x below, hence require
@@ -589,7 +578,7 @@ cmmMachOpFoldM mop args@[x, y@(CmmLit (CmmInt n _))]
                 -- dividend if it is a negative number.
                 --
                 -- to avoid a test/jump, we use the following sequence:
-                -- 	x1 = x >> word_size-1  (all 1s if -ve, all 0s if +ve)
+                --      x1 = x >> word_size-1  (all 1s if -ve, all 0s if +ve)
                 --      x2 = y & (divisor-1)
                 --      result = (x+x2) >>= log2(divisor)
                 -- this could be done a bit more simply using conditional moves,
@@ -606,13 +595,12 @@ cmmMachOpFoldM mop args@[x, y@(CmmLit (CmmInt n _))]
                          CmmMachOp (MO_And rep) [x1, CmmLit (CmmInt (n-1) rep)]
                     x3 = CmmMachOp (MO_Add rep) [x, x2]
                 in
-                Just (cmmMachOpFold (MO_S_Shr rep) [x3, CmmLit (CmmInt p rep)])
-        other
-           -> Nothing
+                Just (cmmMachOpFold platform (MO_S_Shr rep) [x3, CmmLit (CmmInt p rep)])
+        _ -> Nothing
 
 -- Anything else is just too hard.
 
-cmmMachOpFoldM _ _ = Nothing
+cmmMachOpFoldM _ _ _ = Nothing
 
 -- -----------------------------------------------------------------------------
 -- exactLog2
@@ -637,9 +625,9 @@ exactLog2 x_
     else
        case iUnbox (fromInteger x_) of { x ->
        if (x `bitAndFastInt` negateFastInt x) /=# x then
-	  Nothing
+          Nothing
        else
-	  Just (toInteger (iBox (pow2 x)))
+          Just (toInteger (iBox (pow2 x)))
        }
   where
     pow2 x | x ==# _ILIT(1) = _ILIT(0)
@@ -672,34 +660,36 @@ exactLog2 x_
   except factorial, but what the hell.
 -}
 
-cmmLoopifyForC :: RawCmmTop -> RawCmmTop
+cmmLoopifyForC :: RawCmmDecl -> RawCmmDecl
 cmmLoopifyForC p@(CmmProc Nothing _ _) = p  -- only if there's an info table, ignore case alts
-cmmLoopifyForC p@(CmmProc (Just info@(Statics info_lbl _)) entry_lbl
+cmmLoopifyForC (CmmProc (Just info@(Statics info_lbl _)) entry_lbl
                  (ListGraph blocks@(BasicBlock top_id _ : _))) =
 --  pprTrace "jump_lbl" (ppr jump_lbl <+> ppr entry_lbl) $
   CmmProc (Just info) entry_lbl (ListGraph blocks')
   where blocks' = [ BasicBlock id (map do_stmt stmts)
-		  | BasicBlock id stmts <- blocks ]
+                  | BasicBlock id stmts <- blocks ]
 
         do_stmt (CmmJump (CmmLit (CmmLabel lbl)) _) | lbl == jump_lbl
-		= CmmBranch top_id
-	do_stmt stmt = stmt
+                = CmmBranch top_id
+        do_stmt stmt = stmt
 
-	jump_lbl | tablesNextToCode = info_lbl
-		 | otherwise        = entry_lbl
+        jump_lbl | tablesNextToCode = info_lbl
+                 | otherwise        = entry_lbl
 
 cmmLoopifyForC top = top
 
 -- -----------------------------------------------------------------------------
 -- Utils
 
+isLit :: CmmExpr -> Bool
 isLit (CmmLit _) = True
 isLit _          = False
 
 isComparisonExpr :: CmmExpr -> Bool
 isComparisonExpr (CmmMachOp op _) = isComparisonMachOp op
-isComparisonExpr _other 	    = False
+isComparisonExpr _                  = False
 
+isPicReg :: CmmExpr -> Bool
 isPicReg (CmmReg (CmmGlobal PicBaseReg)) = True
 isPicReg _ = False
 
